@@ -33,6 +33,7 @@
 
 #include "cpl_port.h"
 #include "gribdataset.h"
+#include "gdal_priv_templates.hpp"
 
 #include <limits>
 
@@ -182,6 +183,7 @@ GRIB2Section3Writer::GRIB2Section3Writer( VSILFILE* fpIn,
     fp(fpIn),
     poSrcDS(poSrcDSIn)
 {
+    oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     oSRS.SetFromUserInput( poSrcDS->GetProjectionRef() );
     pszProjection = oSRS.GetAttrValue("PROJECTION");
 
@@ -311,6 +313,7 @@ bool GRIB2Section3Writer::TransformToGeo(double& dfX, double& dfY)
 {
     OGRSpatialReference oLL;  // Construct the "geographic" part of oSRS.
     oLL.CopyGeogCSFrom(&oSRS);
+    oLL.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     OGRCoordinateTransformation *poTransformSRSToLL =
         OGRCreateCoordinateTransformation( &(oSRS), &(oLL));
     if( poTransformSRSToLL == nullptr ||
@@ -700,6 +703,7 @@ class GRIB2Section567Writer
         bool Write(float fValOffset,
                    char** papszOptions,
                    GDALProgressFunc pfnProgress, void * pProgressData);
+        void WriteComplexPackingNoData();
 };
 
 /************************************************************************/
@@ -786,7 +790,7 @@ float* GRIB2Section567Writer::GetFloatData()
         m_fMin = m_fMax = static_cast<float>(m_dfNoData);
     }
 
-    // We chech that the actual range of values got from the above RasterIO
+    // We check that the actual range of values got from the above RasterIO
     // request does not go over the expected range of the datatype, as we
     // later assume that for computing nMaxBitsPerElt. 
     // This shouldn't happen for well-behaved drivers, but this can still
@@ -926,6 +930,33 @@ bool GRIB2Section567Writer::WriteSimplePacking()
     return true;
 }
 
+/************************************************************************/
+/*                      WriteComplexPackingNoData()                     */
+/************************************************************************/
+
+void GRIB2Section567Writer::WriteComplexPackingNoData()
+{
+    if( !m_bHasNoData )
+    {
+        WriteUInt32(m_fp, GRIB2MISSING_u4);
+    }
+    else if( GDALDataTypeIsFloating(m_eDT) )
+    {
+        WriteFloat32(m_fp, static_cast<float>(m_dfNoData));
+    }
+    else
+    {
+        if( GDALIsValueInRange<int>(m_dfNoData) )
+        {
+            WriteInt32(m_fp, static_cast<int>(m_dfNoData));
+        }
+        else
+        {
+            WriteUInt32(m_fp, GRIB2MISSING_u4);
+        }
+    }
+}
+
 /************************************************************************/
 /*                       WriteComplexPacking()                          */
 /************************************************************************/
@@ -965,10 +996,7 @@ bool GRIB2Section567Writer::WriteComplexPacking(int nSpatialDifferencingOrder)
         WriteByte(m_fp, GDALDataTypeIsFloating(m_eDT) ? 0 : 1);
         WriteByte(m_fp, 0);
         WriteByte(m_fp, m_bHasNoData ? 1 : 0); // 1 missing value
-        if( !m_bHasNoData )
-            WriteUInt32(m_fp, GRIB2MISSING_u4);
-        else
-            WriteFloat32(m_fp, fNoData);
+        WriteComplexPackingNoData();
         WriteUInt32(m_fp, GRIB2MISSING_u4);
         WriteUInt32(m_fp, 0);
         WriteByte(m_fp, 0);
@@ -1092,10 +1120,7 @@ bool GRIB2Section567Writer::WriteComplexPacking(int nSpatialDifferencingOrder)
     WriteByte(m_fp, GDALDataTypeIsFloating(m_eDT) ? 0 : 1);
     WriteByte(m_fp, idrstmpl[TMPL5_GROUP_SPLITTING_IDX]);
     WriteByte(m_fp, idrstmpl[TMPL5_MISSING_VALUE_MGNT_IDX]);
-    if( !m_bHasNoData )
-        WriteUInt32(m_fp, GRIB2MISSING_u4);
-    else
-        WriteFloat32(m_fp, fNoData);
+    WriteComplexPackingNoData();
     WriteUInt32(m_fp, GRIB2MISSING_u4);
     WriteUInt32(m_fp, idrstmpl[TMPL5_NG_IDX]);
     WriteByte(m_fp, idrstmpl[TMPL5_REF_GROUP_WIDTHS_IDX]);
@@ -2101,7 +2126,7 @@ static float ComputeValOffset(int nTokens, char** papszTokens,
                 fValOffset = 273.15f;
                 CPLDebug("GRIB",
                          "Applying a %f offset to convert from "
-                         "Celcius to Kelvin",
+                         "Celsius to Kelvin",
                          fValOffset);
             }
             else if( !EQUAL(pszInputUnit, "K") )
