@@ -8,7 +8,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam <warmerdam@pobox.com>
- * Copyright (c) 2008-2013, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2018, Even Rouault <even.rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -31,20 +31,9 @@
 
 #include "cpl_port.h"
 
-#define H5_USE_16_API
-
-#ifdef _MSC_VER
-#pragma warning(push)
-// Warning C4005: '_HDF5USEDLL_' : macro redefinition.
-#pragma warning(disable : 4005)
-#endif
-
-#include "hdf5.h"
+#include "hdf5_api.h"
 #include "hdf5dataset.h"
-
-#ifdef _MSC_VER
-#pragma warning(pop)
-#endif
+#include "hdf5vfl.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -56,12 +45,38 @@
 #include "gdal.h"
 #include "gdal_frmts.h"
 #include "gdal_priv.h"
-#
 
-CPL_CVSID("$Id: hdf5dataset.cpp f629c4efe77bfb1aea7332f2c658588fe58c1d45 2018-05-13 23:13:35 +0200 Even Rouault $")
+CPL_CVSID("$Id: hdf5dataset.cpp 6cb6727cae8ceeb5c7321307f2194d08eb4b6201 2019-07-18 00:05:09 +0200 Even Rouault $")
 
 constexpr size_t MAX_METADATA_LEN = 32768;
 
+/************************************************************************/
+/*                          HDF5GetFileDriver()                         */
+/************************************************************************/
+
+hid_t HDF5GetFileDriver()
+{
+    return HDF5VFLGetFileDriver();
+}
+
+/************************************************************************/
+/*                        HDF5UnloadFileDriver()                        */
+/************************************************************************/
+
+void HDF5UnloadFileDriver()
+{
+    HDF5VFLUnloadFileDriver();
+}
+
+/************************************************************************/
+/*                     HDF5DatasetDriverUnload()                        */
+/************************************************************************/
+
+static void HDF5DatasetDriverUnload(GDALDriver*)
+{
+    HDF5UnloadFileDriver();
+}
+
 /************************************************************************/
 /* ==================================================================== */
 /*                              HDF5Dataset                             */
@@ -86,9 +101,11 @@ void GDALRegister_HDF5()
     poDriver->SetMetadataItem(GDAL_DMD_HELPTOPIC, "frmt_hdf5.html");
     poDriver->SetMetadataItem(GDAL_DMD_EXTENSIONS, "h5 hdf5");
     poDriver->SetMetadataItem(GDAL_DMD_SUBDATASETS, "YES");
+    poDriver->SetMetadataItem( GDAL_DCAP_VIRTUALIO, "YES" );
 
     poDriver->pfnOpen = HDF5Dataset::Open;
     poDriver->pfnIdentify = HDF5Dataset::Identify;
+    poDriver->pfnUnloadDriver = HDF5DatasetDriverUnload;
     GetGDALDriverManager()->RegisterDriver(poDriver);
 
 #ifdef HDF5_PLUGIN
@@ -122,6 +139,7 @@ HDF5Dataset::~HDF5Dataset()
         H5Gclose(hGroupID);
     if( hHDF5 > 0 )
         H5Fclose(hHDF5);
+
     CSLDestroy(papszSubDatasets);
     if( poH5RootGroup != nullptr )
     {
@@ -193,11 +211,17 @@ GDALDataType HDF5Dataset::GetDataType(hid_t TypeID)
             return GDT_Unknown;
 
         //For complex the native types of both elements should be the same
-        if ( H5Tequal( H5Tget_member_type(TypeID,0), H5Tget_member_type(TypeID,1)) <= 0 )
+        hid_t ElemTypeID = H5Tget_member_type(TypeID, 0);
+        hid_t Elem2TypeID = H5Tget_member_type(TypeID, 1);
+        const bool bTypeEqual = H5Tequal( ElemTypeID, Elem2TypeID) > 0;
+        H5Tclose(Elem2TypeID);
+        if ( !bTypeEqual )
+        {
+            H5Tclose(ElemTypeID);
             return GDT_Unknown;
+        }
 
         //Check the native types to determine CInt16, CFloat32 or CFloat64
-        hid_t ElemTypeID = H5Tget_member_type(TypeID, 0);
         GDALDataType eDataType = GDT_Unknown;
 
         if ( H5Tequal(H5T_NATIVE_SHORT, ElemTypeID) )
@@ -272,12 +296,17 @@ const char *HDF5Dataset::GetDataTypeName(hid_t TypeID)
             return "Unknown";
 
         //For complex the native types of both elements should be the same
-        if ( H5Tequal( H5Tget_member_type(TypeID,0), H5Tget_member_type(TypeID,1)) <= 0 )
+        hid_t ElemTypeID = H5Tget_member_type(TypeID, 0);
+        hid_t Elem2TypeID = H5Tget_member_type(TypeID, 1);
+        const bool bTypeEqual = H5Tequal( ElemTypeID, Elem2TypeID) > 0;
+        H5Tclose(Elem2TypeID);
+        if ( !bTypeEqual )
+        {
+            H5Tclose(ElemTypeID);
             return "Unknown";
+        }
 
         //Check the native types to determine CInt16, CFloat32 or CFloat64
-        hid_t ElemTypeID = H5Tget_member_type(TypeID, 0);
-        
         if ( H5Tequal(H5T_NATIVE_SHORT, ElemTypeID) )
         {
             H5Tclose(ElemTypeID);
@@ -389,7 +418,11 @@ GDALDataset *HDF5Dataset::Open( GDALOpenInfo *poOpenInfo )
     poDS->SetDescription(poOpenInfo->pszFilename);
 
     // Try opening the dataset.
-    poDS->hHDF5 = H5Fopen(poOpenInfo->pszFilename, H5F_ACC_RDONLY, H5P_DEFAULT);
+    hid_t fapl = H5Pcreate(H5P_FILE_ACCESS);
+    H5Pset_driver(fapl, HDF5GetFileDriver(), nullptr);
+    poDS->hHDF5 = H5Fopen(poOpenInfo->pszFilename, H5F_ACC_RDONLY, fapl);
+    H5Pclose(fapl);
+
     if( poDS->hHDF5 < 0 )
     {
         delete poDS;
@@ -776,7 +809,13 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
     const hid_t hAttrSpace = H5Aget_space(hAttrID);
 
     if( H5Tget_class(hAttrNativeType) == H5T_VLEN )
+    {
+        H5Sclose(hAttrSpace);
+        H5Tclose(hAttrNativeType);
+        H5Tclose(hAttrTypeID);
+        H5Aclose(hAttrID);
         return 0;
+    }
 
     hsize_t nSize[64] = {};
     const unsigned int nAttrDims =
@@ -803,11 +842,11 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
             H5Aread(hAttrID, hAttrNativeType, papszStrings);
 
             // Concatenate all values as one string separated by a space.
-            CPLString osVal = papszStrings[0];
+            CPLString osVal = papszStrings[0] ? papszStrings[0] : "{NULL}";
             for( hsize_t i = 1; i < nAttrElmts; i++ )
             {
                 osVal += " ";
-                osVal += papszStrings[i];
+                osVal += papszStrings[i] ? papszStrings[i] : "{NULL}";
             }
 
             szValue = static_cast<char *>(CPLMalloc(osVal.length() + 1));
@@ -839,30 +878,48 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
             szValue[0] = '\0';
             H5Aread(hAttrID, hAttrNativeType, buf);
         }
-        if( H5Tequal(H5T_NATIVE_CHAR, hAttrNativeType ) ||
-            H5Tequal(H5T_NATIVE_SCHAR, hAttrNativeType) )
+        const bool bIsSCHAR = H5Tequal(H5T_NATIVE_SCHAR, hAttrNativeType) > 0;
+        const bool bIsUCHAR = H5Tequal(H5T_NATIVE_UCHAR, hAttrNativeType) > 0;
+        if( (bIsSCHAR || bIsUCHAR) &&
+            CPLTestBool(CPLGetConfigOption("GDAL_HDF5_CHAR_AS_STRING", "NO")) )
         {
+            // Compatibility mode with ancient GDAL versions where we consider
+            // array of SCHAR/UCHAR as strings. Likely inappropriate mode...
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
-                snprintf(szData, nDataLen, "%c ", static_cast<char *>(buf)[i]);
+                snprintf(szData, nDataLen, "%c",
+                         static_cast<char *>(buf)[i]);
                 if( CPLStrlcat(szValue, szData, MAX_METADATA_LEN) >=
                     MAX_METADATA_LEN )
                     CPLError(CE_Warning, CPLE_OutOfMemory,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_UCHAR, hAttrNativeType) )
+        else if( bIsSCHAR )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
-                snprintf(szData, nDataLen, "%c", static_cast<char *>(buf)[i]);
+                snprintf(szData, nDataLen, "%d ",
+                         static_cast<signed char *>(buf)[i]);
                 if( CPLStrlcat(szValue, szData, MAX_METADATA_LEN) >=
                     MAX_METADATA_LEN )
                     CPLError(CE_Warning, CPLE_OutOfMemory,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_SHORT, hAttrNativeType) )
+        else if( bIsUCHAR )
+        {
+            for( hsize_t i = 0; i < nAttrElmts; i++ )
+            {
+                snprintf(szData, nDataLen, "%u ",
+                         static_cast<unsigned char *>(buf)[i]);
+                if( CPLStrlcat(szValue, szData, MAX_METADATA_LEN) >=
+                    MAX_METADATA_LEN )
+                    CPLError(CE_Warning, CPLE_OutOfMemory,
+                             "Header data too long. Truncated");
+            }
+        }
+        else if( H5Tequal(H5T_NATIVE_SHORT, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
@@ -873,11 +930,11 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_USHORT, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_USHORT, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
-              snprintf(szData, nDataLen, "%ud ",
+              snprintf(szData, nDataLen, "%u ",
                        static_cast<unsigned short *>(buf)[i]);
                 if( CPLStrlcat(szValue, szData, MAX_METADATA_LEN) >=
                     MAX_METADATA_LEN )
@@ -885,7 +942,7 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_INT, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_INT, hAttrNativeType) > 0 )
         {
             for( hsize_t i=0; i < nAttrElmts; i++ )
             {
@@ -896,11 +953,11 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_UINT, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_UINT, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
-                snprintf(szData, nDataLen, "%ud ",
+                snprintf(szData, nDataLen, "%u ",
                          static_cast<unsigned int *>(buf)[i]);
                 if( CPLStrlcat(szValue, szData, MAX_METADATA_LEN) >=
                     MAX_METADATA_LEN )
@@ -908,7 +965,7 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_LONG, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_LONG, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
@@ -919,7 +976,7 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_ULONG, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_ULONG, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ ) {
                 snprintf(szData, nDataLen, "%lu ",
@@ -930,7 +987,7 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_FLOAT, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_FLOAT, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
@@ -942,7 +999,7 @@ static herr_t HDF5AttrIterate( hid_t hH5ObjID,
                              "Header data too long. Truncated");
             }
         }
-        else if( H5Tequal(H5T_NATIVE_DOUBLE, hAttrNativeType) )
+        else if( H5Tequal(H5T_NATIVE_DOUBLE, hAttrNativeType) > 0 )
         {
             for( hsize_t i = 0; i < nAttrElmts; i++ )
             {
