@@ -33,7 +33,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: pcidskdataset2.cpp 22f8ae3bf7bc3cccd970992655c63fc5254d3206 2018-04-08 20:13:05 +0200 Even Rouault $")
+CPL_CVSID("$Id: pcidskdataset2.cpp 3189229c71a9620126f6b349f4f80399baeaf528 2019-04-20 20:33:36 +0200 Even Rouault $")
 
 const PCIDSK::PCIDSKInterfaces *PCIDSK2GetInterfaces(void);
 
@@ -517,8 +517,9 @@ void PCIDSK2Band::RefreshOverviewList()
 /* -------------------------------------------------------------------- */
     for( int iOver = 0; iOver < poChannel->GetOverviewCount(); iOver++ )
     {
-        apoOverviews.push_back(
-            new PCIDSK2Band( poChannel->GetOverview(iOver) ) );
+        auto poOvrBand = new PCIDSK2Band( poChannel->GetOverview(iOver) );
+        poOvrBand->eAccess = eAccess;
+        apoOverviews.push_back( poOvrBand );
     }
 }
 
@@ -710,7 +711,7 @@ char **PCIDSK2Band::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALPamRasterBand::GetMetadataDomainList(),
                                    TRUE,
-                                   "", NULL);
+                                   "", nullptr);
 }
 
 /************************************************************************/
@@ -828,6 +829,9 @@ PCIDSK2Dataset::~PCIDSK2Dataset()
         apoLayers.pop_back();
     }
 
+    if( m_poSRS )
+        m_poSRS->Release();
+
     try {
         if( poFile != nullptr)
             delete poFile;
@@ -1144,7 +1148,7 @@ char **PCIDSK2Dataset::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALPamDataset::GetMetadataDomainList(),
                                    TRUE,
-                                   "", NULL);
+                                   "", nullptr);
 }
 
 /************************************************************************/
@@ -1331,14 +1335,12 @@ CPLErr PCIDSK2Dataset::GetGeoTransform( double * padfTransform )
 }
 
 /************************************************************************/
-/*                           SetProjection()                            */
+/*                           SetSpatialRef()                            */
 /************************************************************************/
 
-CPLErr PCIDSK2Dataset::SetProjection( const char *pszWKT )
+CPLErr PCIDSK2Dataset::SetSpatialRef( const OGRSpatialReference* poSRS )
 
 {
-    osSRS = "";
-
     PCIDSKGeoref *poGeoref = nullptr;
 
     try
@@ -1353,20 +1355,18 @@ CPLErr PCIDSK2Dataset::SetProjection( const char *pszWKT )
 
     if( poGeoref == nullptr )
     {
-        return GDALPamDataset::SetProjection( pszWKT );
+        return GDALPamDataset::SetSpatialRef( poSRS );
     }
 
     char *pszGeosys = nullptr;
     char *pszUnits = nullptr;
     double *padfPrjParams = nullptr;
 
-    OGRSpatialReference oSRS;
-
-    if( ! (oSRS.importFromWkt(pszWKT ) == OGRERR_NONE
-        && oSRS.exportToPCI( &pszGeosys, &pszUnits,
-                             &padfPrjParams ) == OGRERR_NONE ) )
+    if( poSRS == nullptr ||
+        poSRS->exportToPCI( &pszGeosys, &pszUnits,
+                             &padfPrjParams ) != OGRERR_NONE )
     {
-        return GDALPamDataset::SetProjection( pszWKT );
+        return GDALPamDataset::SetSpatialRef( poSRS );
     }
 
     if( GetAccess() == GA_ReadOnly )
@@ -1427,13 +1427,13 @@ CPLErr PCIDSK2Dataset::SetProjection( const char *pszWKT )
 }
 
 /************************************************************************/
-/*                          GetProjectionRef()                          */
+/*                          GetSpatialRef()                             */
 /************************************************************************/
 
-const char *PCIDSK2Dataset::GetProjectionRef()
+const OGRSpatialReference *PCIDSK2Dataset::GetSpatialRef() const
 {
-    if( osSRS != "" )
-        return osSRS.c_str();
+    if( m_poSRS )
+        return m_poSRS;
 
     PCIDSKGeoref *poGeoref = nullptr;
 
@@ -1449,8 +1449,7 @@ const char *PCIDSK2Dataset::GetProjectionRef()
 
     if( poGeoref == nullptr )
     {
-        osSRS = GDALPamDataset::GetProjectionRef();
-        return osSRS.c_str();
+        return GDALPamDataset::GetSpatialRef();
     }
 
     CPLString osGeosys;
@@ -1482,20 +1481,17 @@ const char *PCIDSK2Dataset::GetProjectionRef()
     }
 
     OGRSpatialReference oSRS;
+    oSRS.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
     if( oSRS.importFromPCI( osGeosys, pszUnits,
                             &(adfParameters[0]) ) == OGRERR_NONE )
     {
-        char *pszWKT = nullptr;
-        oSRS.exportToWkt( &pszWKT );
-        osSRS = pszWKT;
-        CPLFree( pszWKT );
+        m_poSRS = oSRS.Clone();
+        return m_poSRS;
     }
     else
     {
-        osSRS = GDALPamDataset::GetProjectionRef();
+        return GDALPamDataset::GetSpatialRef();
     }
-
-    return osSRS.c_str();
 }
 
 /************************************************************************/
@@ -1509,6 +1505,48 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
                                         void *pProgressData )
 
 {
+    PCIDSK2Band *poBand = reinterpret_cast<PCIDSK2Band*>(
+        GetRasterBand( panBandList[0] ) );
+
+/* -------------------------------------------------------------------- */
+/*      If RRD overviews requested, then invoke generic handling.       */
+/* -------------------------------------------------------------------- */
+    bool bUseGenericHandling = false;
+
+    if( CPLTestBool(CPLGetConfigOption( "USE_RRD", "NO" ) ) )
+    {
+        bUseGenericHandling = true;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      If we don't have read access, then create the overviews         */
+/*      externally.                                                     */
+/* -------------------------------------------------------------------- */
+    if( GetAccess() != GA_Update )
+    {
+        CPLDebug( "PCIDSK",
+                  "File open for read-only accessing, "
+                  "creating overviews externally." );
+
+        bUseGenericHandling = true;
+    }
+
+    if( bUseGenericHandling )
+    {
+        if( poBand->GetOverviewCount() != 0 )
+        {
+            CPLError(
+                CE_Failure, CPLE_NotSupported,
+                "Cannot add external overviews when there are already "
+                "internal overviews" );
+            return CE_Failure;
+        }
+
+        return GDALDataset::IBuildOverviews(
+            pszResampling, nOverviews, panOverviewList,
+            nListBands, panBandList, pfnProgress, pProgressData );
+    }
+
     if( nListBands == 0 )
         return CE_None;
 
@@ -1527,8 +1565,6 @@ CPLErr PCIDSK2Dataset::IBuildOverviews( const char *pszResampling,
 /*      which are new.  We assume that band 1 of the file is            */
 /*      representative.                                                 */
 /* -------------------------------------------------------------------- */
-    PCIDSK2Band *poBand = reinterpret_cast<PCIDSK2Band*>(
-        GetRasterBand( panBandList[0] ) );
 
     int nNewOverviews = 0;
     int *panNewOverviewList = reinterpret_cast<int *>(
