@@ -34,7 +34,7 @@
 #include <cmath>
 #include <cstdlib>
 
-CPL_CVSID("$Id: btdataset.cpp 6574497e5ddfd7c08c094a76756a0ef477cef6a1 2018-04-04 22:15:20 +0200 Even Rouault $")
+CPL_CVSID("$Id: btdataset.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -42,7 +42,7 @@ CPL_CVSID("$Id: btdataset.cpp 6574497e5ddfd7c08c094a76756a0ef477cef6a1 2018-04-0
 /* ==================================================================== */
 /************************************************************************/
 
-class BTDataset : public GDALPamDataset
+class BTDataset final: public GDALPamDataset
 {
     friend class BTRasterBand;
 
@@ -60,12 +60,20 @@ class BTDataset : public GDALPamDataset
 
     float        m_fVscale;
 
+    CPL_DISALLOW_COPY_ASSIGN(BTDataset)
+
   public:
     BTDataset();
     ~BTDataset() override;
 
-    const char *GetProjectionRef(void) override;
-    CPLErr SetProjection( const char * ) override;
+    const char *_GetProjectionRef(void) override;
+    const OGRSpatialReference* GetSpatialRef() const override {
+        return GetSpatialRefFromOldGetProjectionRef();
+    }
+    CPLErr _SetProjection( const char * ) override;
+    CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
+        return OldSetProjectionFromSetSpatialRef(poSRS);
+    }
     CPLErr GetGeoTransform( double * ) override;
     CPLErr SetGeoTransform( double * ) override;
 
@@ -83,10 +91,12 @@ class BTDataset : public GDALPamDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class BTRasterBand : public GDALPamRasterBand
+class BTRasterBand final: public GDALPamRasterBand
 {
     VSILFILE          *fpImage;
 
+    CPL_DISALLOW_COPY_ASSIGN(BTRasterBand)
+
   public:
                    BTRasterBand( GDALDataset * poDS, VSILFILE * fp,
                                  GDALDataType eType );
@@ -301,7 +311,7 @@ static bool approx_equals( float a, float b )
 
 const char* BTRasterBand::GetUnitType(void)
 {
-    const BTDataset& ds = *(BTDataset*)poDS;
+    const BTDataset& ds = * cpl::down_cast<const BTDataset*>(poDS);
     float f = ds.m_fVscale;
     if(f == 1.0f)
         return "m";
@@ -325,7 +335,7 @@ const char* BTRasterBand::GetUnitType(void)
 
 CPLErr BTRasterBand::SetUnitType(const char* psz)
 {
-    BTDataset& ds = *(BTDataset*)poDS;
+    BTDataset& ds = * cpl::down_cast<BTDataset*>(poDS);
     if(EQUAL(psz, "m"))
         ds.m_fVscale = 1.0f;
     else if(EQUAL(psz, "ft"))
@@ -380,7 +390,7 @@ BTDataset::BTDataset() :
 BTDataset::~BTDataset()
 
 {
-    FlushCache();
+    BTDataset::FlushCache();
     if( fpImage != nullptr )
     {
         if( VSIFCloseL( fpImage ) != 0 )
@@ -471,7 +481,7 @@ CPLErr BTDataset::SetGeoTransform( double *padfTransform )
 /*                          GetProjectionRef()                          */
 /************************************************************************/
 
-const char *BTDataset::GetProjectionRef()
+const char *BTDataset::_GetProjectionRef()
 
 {
     if( pszProjection == nullptr )
@@ -484,7 +494,7 @@ const char *BTDataset::GetProjectionRef()
 /*                           SetProjection()                            */
 /************************************************************************/
 
-CPLErr BTDataset::SetProjection( const char *pszNewProjection )
+CPLErr BTDataset::_SetProjection( const char *pszNewProjection )
 
 {
     CPLErr eErr = CE_None;
@@ -528,7 +538,7 @@ CPLErr BTDataset::SetProjection( const char *pszNewProjection )
 /* -------------------------------------------------------------------- */
     int bNorth = FALSE;
 
-    nShortTemp = (GInt16) oSRS.GetUTMZone( &bNorth );
+    nShortTemp = static_cast<GInt16>(oSRS.GetUTMZone( &bNorth ));
     if( bNorth )
         nShortTemp = -nShortTemp;
 
@@ -596,7 +606,7 @@ GDALDataset *BTDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     char szVersion[4] = {};
 
-    strncpy( szVersion, (char *) (poDS->abyHeader + 7), 3 );
+    strncpy( szVersion, reinterpret_cast<char *>(poDS->abyHeader + 7), 3 );
     szVersion[3] = '\0';
     poDS->nVersionCode = static_cast<int>(CPLAtof(szVersion) * 10);
 
@@ -925,8 +935,8 @@ GDALDataset *BTDataset::Create( const char * pszFilename,
 /* -------------------------------------------------------------------- */
 /*      Write to disk.                                                  */
 /* -------------------------------------------------------------------- */
-    if( VSIFWriteL( (void *) abyHeader, 256, 1, fp ) != 1 ||
-        VSIFSeekL( fp, (GDALGetDataTypeSize(eType)/8) * nXSize * (vsi_l_offset)nYSize - 1,
+    if( VSIFWriteL( abyHeader, 256, 1, fp ) != 1 ||
+        VSIFSeekL( fp, (GDALGetDataTypeSize(eType)/8) * nXSize * static_cast<vsi_l_offset>(nYSize) - 1,
                    SEEK_CUR ) != 0
         || VSIFWriteL( abyHeader+255, 1, 1, fp ) != 1 )
     {
@@ -948,7 +958,7 @@ GDALDataset *BTDataset::Create( const char * pszFilename,
         return nullptr;
     }
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+    return GDALDataset::Open( pszFilename, GDAL_OF_RASTER | GDAL_OF_UPDATE );
 }
 
 /************************************************************************/
