@@ -46,18 +46,9 @@
 #endif  // !DEBUG_VERBOSE
 
 #include "ogr_geojson.h"
-
-// Remove annoying warnings Microsoft Visual C++:
-//   'class': assignment operator could not be generated.
-//     The compiler cannot generate an assignment operator for the given
-//     class. No assignment operator was created.
-#if defined(_MSC_VER)
-#  pragma warning(disable:4512)
-#endif
-
 #include "ogrgeojsonreader.h"
 
-CPL_CVSID("$Id: ogrgeojsonlayer.cpp c7a85063f3aed63008861e94f030482ed2ce7177 2018-06-02 11:57:52 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrgeojsonlayer.cpp 07b24f8cd487508a42b6ca5df2e5b96a85e9f204 2019-06-27 22:53:04 +0200 Even Rouault $")
 
 /************************************************************************/
 /*                       STATIC MEMBERS DEFINITION                      */
@@ -137,6 +128,7 @@ void OGRGeoJSONLayer::SetFIDColumn( const char* pszFIDColumn )
 
 void OGRGeoJSONLayer::ResetReading()
 {
+    nFeatureReadSinceReset_ = 0;
     if( poReader_ )
     {
         TerminateAppendSession();
@@ -174,6 +166,7 @@ OGRFeature* OGRGeoJSONLayer::GetNextFeature()
                 && (m_poAttrQuery == nullptr ||
                     m_poAttrQuery->Evaluate(poFeature)) )
             {
+                nFeatureReadSinceReset_ ++;
                 return poFeature;
             }
             delete poFeature;
@@ -181,7 +174,12 @@ OGRFeature* OGRGeoJSONLayer::GetNextFeature()
     }
     else
     {
-        return OGRMemLayer::GetNextFeature();
+        auto ret = OGRMemLayer::GetNextFeature();
+        if( ret )
+        {
+            nFeatureReadSinceReset_ ++;
+        }
+        return ret;
     }
 }
 
@@ -214,6 +212,10 @@ OGRFeature* OGRGeoJSONLayer::GetFeature(GIntBig nFID)
 {
     if( poReader_ )
     {
+        if( !IsUpdatable() )
+        {
+            return poReader_->GetFeature(this, nFID);
+        }
         return OGRLayer::GetFeature(nFID);
     }
     else
@@ -253,8 +255,15 @@ bool OGRGeoJSONLayer::IngestAll()
 
 OGRErr OGRGeoJSONLayer::ISetFeature( OGRFeature *poFeature )
 {
-    if( !IsUpdatable() || !IngestAll() )
+    if( !IsUpdatable() )
         return OGRERR_FAILURE;
+    if( poReader_ )
+    {
+        auto nNextIndex = nFeatureReadSinceReset_;
+        if( !IngestAll() )
+            return OGRERR_FAILURE;
+        SetNextByIndex(nNextIndex);
+    }
     return OGRMemLayer::ISetFeature(poFeature);
 }
 
@@ -534,17 +543,9 @@ void OGRGeoJSONLayer::DetectGeometryType()
         if( nullptr != poGeometry )
         {
             OGRwkbGeometryType eGeomType = poGeometry->getGeometryType();
-            if( bFirstGeometry )
+            if( !OGRGeoJSONUpdateLayerGeomType(
+                    this, bFirstGeometry, eGeomType, eLayerGeomType) )
             {
-                eLayerGeomType = eGeomType;
-                GetLayerDefn()->SetGeomType( eGeomType );
-                bFirstGeometry = false;
-            }
-            else if( eGeomType != eLayerGeomType )
-            {
-                CPLDebug( "GeoJSON",
-                    "Detected layer of mixed-geometry type features." );
-                GetLayerDefn()->SetGeomType( DefaultGeometryType );
                 delete poFeature;
                 break;
             }
