@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_geopackage.h 3b495c65e84e938192dead1d28cc7c6552c3149e 2018-06-14 16:21:05 +0200 Even Rouault $
+ * $Id: ogr_geopackage.h 1f5037da497fb5ee7497768ccf4172b92a91578d 2019-02-19 21:19:51 +0100 Even Rouault $
  *
  * Project:  GeoPackage Translator
  * Purpose:  Definition of classes for OGR GeoPackage driver.
@@ -98,7 +98,6 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
     GUInt32             m_nUserVersion;
     OGRGeoPackageTableLayer** m_papoLayers;
     int                 m_nLayers;
-    bool                m_bUtf8;
     void                CheckUnknownExtensions(bool bCheckRasterTable = false);
 #ifdef ENABLE_GPKG_OGR_CONTENTS
     bool                m_bHasGPKGOGRContents;
@@ -133,6 +132,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
     CPLString           m_osTilingScheme;
 
         bool            ComputeTileAndPixelShifts();
+        bool            AllocCachedTiles();
         bool            InitRaster ( GDALGeoPackageDataset* poParentDS,
                                      const char* pszTableName,
                                      double dfMinX,
@@ -192,6 +192,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
 
         int                     FindLayerIndex(const char* pszLayerName);
 
+        bool                    HasGriddedCoverageAncillaryTable();
         bool                    CreateTileGriddedTable(char** papszOptions);
 
         void                    CreateOGREmptyTableIfNeeded();
@@ -205,7 +206,7 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         bool                    m_bMapTableToExtensionsBuilt;
         std::map< CPLString, std::vector<GPKGExtensionDesc> > m_oMapTableToExtensions;
         const std::map< CPLString, std::vector<GPKGExtensionDesc> > &
-                                        GetExtensions();
+                                        GetUnknownExtensionsTableSpecific();
 
         bool                    m_bMapTableToContentsBuilt;
         std::map< CPLString, GPKGContentsDesc > m_oMapTableToContents;
@@ -218,6 +219,8 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         bool                DeleteVectorOrRasterLayer(
                                                 const char* pszLayerName );
 
+        bool                ConvertGpkgSpatialRefSysToExtensionWkt2();
+
     public:
                             GDALGeoPackageDataset();
                             virtual ~GDALGeoPackageDataset();
@@ -232,8 +235,14 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
                                              const char * pszValue,
                                              const char * pszDomain = "" ) override;
 
-        virtual const char* GetProjectionRef() override;
-        virtual CPLErr      SetProjection( const char* pszProjection ) override;
+        virtual const char* _GetProjectionRef() override;
+        virtual CPLErr      _SetProjection( const char* pszProjection ) override;
+        const OGRSpatialReference* GetSpatialRef() const override {
+            return GetSpatialRefFromOldGetProjectionRef();
+        }
+        CPLErr SetSpatialRef(const OGRSpatialReference* poSRS) override {
+            return OldSetProjectionFromSetSpatialRef(poSRS);
+        }
 
         virtual CPLErr      GetGeoTransform( double* padfGeoTransform ) override;
         virtual CPLErr      SetGeoTransform( double* padfGeoTransform ) override;
@@ -273,13 +282,14 @@ class GDALGeoPackageDataset final : public OGRSQLiteBaseDataSource, public GDALG
         int                 GetSrsId( const OGRSpatialReference& oSRS );
         const char*         GetSrsName( const OGRSpatialReference& oSRS );
         OGRSpatialReference* GetSpatialRef( int iSrsId );
-        bool                GetUTF8() { return m_bUtf8; }
         OGRErr              CreateExtensionsTableIfNecessary();
         bool                HasExtensionsTable();
         OGRErr              CreateGDALAspatialExtension();
-        bool                HasDataColumnsTable();
         void                SetMetadataDirty() { m_bMetadataDirty = true; }
 
+        bool                    HasDataColumnsTable();
+        bool                    HasDataColumnConstraintsTable();
+
         const char*         GetGeometryTypeString(OGRwkbGeometryType eType);
 
         void                ResetReadingAllLayers();
@@ -451,6 +461,8 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     OGRErr              ReadTableDefinition();
     void                InitView();
 
+    bool                DoSpecialProcessingForColumnCreation(OGRFieldDefn* poField);
+
     public:
                         OGRGeoPackageTableLayer( GDALGeoPackageDataset *poDS,
                                                  const char * pszTableName );
@@ -514,6 +526,8 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     void                CreateSpatialIndexIfNecessary();
     bool                CreateSpatialIndex(const char* pszTableName = nullptr);
     bool                DropSpatialIndex(bool bCalledFromSQLFunction = false);
+    CPLString           ReturnSQLCreateSpatialIndexTriggers(const char* pszTableName);
+    CPLString           ReturnSQLDropSpatialIndexTriggers();
 
     virtual char **     GetMetadata( const char *pszDomain = nullptr ) override;
     virtual const char *GetMetadataItem( const char * pszName,
@@ -567,6 +581,7 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
     OGRErr              FeatureBindUpdateParameters( OGRFeature *poFeature, sqlite3_stmt *poStmt );
     OGRErr              FeatureBindInsertParameters( OGRFeature *poFeature, sqlite3_stmt *poStmt, bool bAddFID, bool bBindUnsetFields );
     OGRErr              FeatureBindParameters( OGRFeature *poFeature, sqlite3_stmt *poStmt, int *pnColCount, bool bAddFID, bool bBindUnsetFields );
+    void                UpdateContentsToNullExtent();
 
     void                CheckUnknownExtensions();
     bool                CreateGeometryExtensionIfNecessary(const OGRGeometry* poGeom);
@@ -579,6 +594,8 @@ class OGRGeoPackageTableLayer final : public OGRGeoPackageLayer
 
 class OGRGeoPackageSelectLayer final : public OGRGeoPackageLayer, public IOGRSQLiteSelectLayer
 {
+    CPL_DISALLOW_COPY_ASSIGN(OGRGeoPackageSelectLayer)
+
     OGRSQLiteSelectLayerCommonBehaviour* poBehaviour;
 
     virtual OGRErr      ResetStatement() override;
