@@ -1,6 +1,6 @@
 /*-*-C++-*-*/
 /******************************************************************************
- * $Id: ogr_tiger.h 002b050d9a9ef403a732c1210784736ef97216d4 2018-04-09 21:34:55 +0200 Even Rouault $
+ * $Id: ogr_tiger.h 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Main declarations for Tiger translator.
@@ -554,6 +554,8 @@ class OGRTigerDataSource : public OGRDataSource
 
     TigerVersion        TigerCheckVersion( TigerVersion, const char * );
 
+    CPL_DISALLOW_COPY_ASSIGN(OGRTigerDataSource)
+
   public:
                         OGRTigerDataSource();
                         virtual ~OGRTigerDataSource();
@@ -579,7 +581,7 @@ class OGRTigerDataSource : public OGRDataSource
     void                AddLayer( OGRTigerLayer * );
     int                 TestCapability( const char * ) override;
 
-    OGRSpatialReference *GetSpatialRef() { return poSpatialRef; }
+    OGRSpatialReference *DSGetSpatialRef() { return poSpatialRef; }
 
     const char          *GetDirPath() { return pszPath; }
     char                *BuildFilename( const char * pszModule,
