@@ -38,7 +38,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: ogrwfsdatasource.cpp 82025de8ed8c31e58bf2adf2f98c2f9b6214ad33 2018-03-30 17:17:49 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrwfsdatasource.cpp 3189229c71a9620126f6b349f4f80399baeaf528 2019-04-20 20:33:36 +0200 Even Rouault $")
 
 constexpr int DEFAULT_BASE_START_INDEX = 0;
 constexpr int DEFAULT_PAGE_SIZE = 100;
@@ -305,7 +305,7 @@ char** OGRWFSDataSource::GetMetadataDomainList()
 {
     return BuildMetadataDomainList(GDALDataset::GetMetadataDomainList(),
                                    TRUE,
-                                   "", "xml:capabilities", NULL);
+                                   "", "xml:capabilities", nullptr);
 }
 
 /************************************************************************/
@@ -1364,19 +1364,12 @@ int OGRWFSDataSource::Open( const char * pszFilename, int bUpdateIn,
                     if (oSRS.SetFromUserInput(pszDefaultSRS) == OGRERR_NONE)
                     {
                         poSRS = oSRS.Clone();
-                        if( bInvertAxisOrderIfLatLong &&
-                            GML_IsSRSLatLongOrder(pszDefaultSRS) )
+                        poSRS->SetAxisMappingStrategy(
+                            bInvertAxisOrderIfLatLong ? OAMS_TRADITIONAL_GIS_ORDER : OAMS_AUTHORITY_COMPLIANT);
+                        if( GML_IsSRSLatLongOrder(pszDefaultSRS) &&
+                            bInvertAxisOrderIfLatLong )
                         {
                             bAxisOrderAlreadyInverted = true;
-
-                            OGR_SRSNode *poGEOGCS =
-                                            poSRS->GetAttrNode( "GEOGCS" );
-                            if( poGEOGCS != nullptr )
-                                poGEOGCS->StripNodes( "AXIS" );
-
-                            OGR_SRSNode *poPROJCS = poSRS->GetAttrNode( "PROJCS" );
-                            if (poPROJCS != nullptr && poSRS->EPSGTreatsAsNorthingEasting())
-                                poPROJCS->StripNodes( "AXIS" );
                         }
                     }
                 }
@@ -1498,8 +1491,7 @@ int OGRWFSDataSource::Open( const char * pszFilename, int bUpdateIn,
                                     "FALSE")));
 
                         if (((bTrustBounds || (dfMinX == -180 && dfMinY == -90 && dfMaxX == 180 && dfMaxY == 90)) &&
-                            (strcmp(pszProj4, "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs ") == 0 ||
-                             strcmp(pszProj4, "+proj=longlat +datum=WGS84 +no_defs ") == 0)) ||
+                            strcmp(pszProj4, "+proj=longlat +datum=WGS84 +no_defs") == 0) ||
                             strcmp(pszDefaultSRS, "urn:ogc:def:crs:OGC:1.3:CRS84") == 0)
                         {
                             poLayer->SetExtents(dfMinX, dfMinY, dfMaxX, dfMaxY);
@@ -1509,6 +1501,7 @@ int OGRWFSDataSource::Open( const char * pszFilename, int bUpdateIn,
                         {
                             OGRSpatialReference oWGS84;
                             oWGS84.SetWellKnownGeogCS("WGS84");
+                            oWGS84.SetAxisMappingStrategy(OAMS_TRADITIONAL_GIS_ORDER);
                             CPLPushErrorHandler(CPLQuietErrorHandler);
                             OGRCoordinateTransformation* poCT =
                                 OGRCreateCoordinateTransformation(&oWGS84,
@@ -1620,7 +1613,7 @@ void OGRWFSDataSource::LoadMultipleLayerDefn(const char* pszLayerName,
     else
         *pszPrefix = 0;
 
-    OGRWFSLayer* poRefLayer = (OGRWFSLayer*)GetLayerByName(pszLayerName);
+    OGRWFSLayer* poRefLayer = dynamic_cast<OGRWFSLayer*>(GetLayerByName(pszLayerName));
     if (poRefLayer == nullptr)
         return;
 
