@@ -34,7 +34,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: ogrxlsxdatasource.cpp 098114356bc7dc6a5303d62e63b079a950ff2f66 2018-05-14 11:29:01 +0200 Even Rouault $")
+CPL_CVSID("$Id: ogrxlsxdatasource.cpp 8e5eeb35bf76390e3134a4ea7076dab7d478ea0e 2018-11-14 22:55:13 +0100 Even Rouault $")
 
 namespace OGRXLSX {
 
@@ -115,7 +115,6 @@ OGRFeature* OGRXLSXLayer::GetNextFeature()
 OGRErr OGRXLSXLayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
 {
     Init();
-    // BuildColString() takes a 4 character string + nul byte
     if( GetLayerDefn()->GetFieldCount() >= 2000 )
     {
         CPLError(CE_Failure, CPLE_AppDefined,
@@ -1002,6 +1001,7 @@ void OGRXLSXDataSource::endElementRow(CPL_UNUSED const char *pszNameIn)
                         else if (eFieldType != OFTString && eValType != eFieldType)
                         {
                             OGRFieldDefn oNewFieldDefn(poFieldDefn);
+                            oNewFieldDefn.SetSubType(OFSTNone);
                             if ((eFieldType == OFTDate || eFieldType == OFTTime) &&
                                    eValType == OFTDateTime)
                                 oNewFieldDefn.SetType(OFTDateTime);
@@ -1676,8 +1676,8 @@ void OGRXLSXDataSource::AnalyseStyles(VSILFILE* fpStyles)
 
 OGRLayer *
 OGRXLSXDataSource::ICreateLayer( const char * pszLayerName,
-                                 CPL_UNUSED OGRSpatialReference *poSRS,
-                                 CPL_UNUSED OGRwkbGeometryType eType,
+                                 OGRSpatialReference * /*poSRS*/,
+                                 OGRwkbGeometryType /*eType*/,
                                  char ** papszOptions )
 
 {
@@ -1944,29 +1944,30 @@ static bool WriteWorkbook(const char* pszName, GDALDataset* poDS)
 /*                            BuildColString()                          */
 /************************************************************************/
 
-static void BuildColString(char szCol[5], int nCol)
+static CPLString BuildColString(int nCol)
 {
     /*
     A Z   AA AZ   BA BZ   ZA   ZZ   AAA    ZZZ      AAAA
     0 25  26 51   52 77   676  701  702    18277    18278
     */
-    int k = 0;
-    szCol[k++] = (nCol % 26) + 'A';
+    CPLString osRet;
+    osRet += (nCol % 26) + 'A';
     while(nCol >= 26)
     {
         nCol /= 26;
         // We would not need a decrement if this was a proper base 26
         // numeration scheme.
         nCol --;
-        szCol[k++] = (nCol % 26) + 'A';
+        osRet += (nCol % 26) + 'A';
     }
-    szCol[k] = 0;
-    for(int l=0;l<k/2;l++)
+    const size_t nSize = osRet.size();
+    for(size_t l=0;l<nSize/2;l++)
     {
-        char chTmp = szCol[k-1-l];
-        szCol[k-1-l] = szCol[l];
-        szCol[l] = chTmp;
+        char chTmp = osRet[nSize-1-l];
+        osRet[nSize-1-l] = osRet[l];
+        osRet[l] = chTmp;
     }
+    return osRet;
 }
 
 /************************************************************************/
@@ -2034,10 +2035,9 @@ static bool WriteLayer(const char* pszName, OGRLayer* poLayer, int iLayer,
                 oStringList.push_back(pszVal);
             }
 
-            char szCol[5];
-            BuildColString(szCol, j);
+            CPLString osCol = BuildColString(j);
 
-            VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"s\">\n", szCol, iRow);
+            VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"s\">\n", osCol.c_str(), iRow);
             VSIFPrintfL(fp, "<v>%d</v>\n", nStringIndex);
             VSIFPrintfL(fp, "</c>\n");
         }
@@ -2053,15 +2053,14 @@ static bool WriteLayer(const char* pszName, OGRLayer* poLayer, int iLayer,
         {
             if (poFeature->IsFieldSetAndNotNull(j))
             {
-                char szCol[5];
-                BuildColString(szCol, j);
+                CPLString osCol = BuildColString(j);
 
                 OGRFieldDefn* poFieldDefn = poFDefn->GetFieldDefn(j);
                 OGRFieldType eType = poFieldDefn->GetType();
 
                 if (eType == OFTReal)
                 {
-                    VSIFPrintfL(fp, "<c r=\"%s%d\">\n", szCol, iRow);
+                    VSIFPrintfL(fp, "<c r=\"%s%d\">\n", osCol.c_str(), iRow);
                     VSIFPrintfL(fp, "<v>%.16g</v>\n", poFeature->GetFieldAsDouble(j));
                     VSIFPrintfL(fp, "</c>\n");
                 }
@@ -2069,15 +2068,15 @@ static bool WriteLayer(const char* pszName, OGRLayer* poLayer, int iLayer,
                 {
                     OGRFieldSubType eSubType = poFieldDefn->GetSubType();
                     if( eSubType == OFSTBoolean )
-                        VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"b\" s=\"5\">\n", szCol, iRow);
+                        VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"b\" s=\"5\">\n", osCol.c_str(), iRow);
                     else
-                        VSIFPrintfL(fp, "<c r=\"%s%d\">\n", szCol, iRow);
+                        VSIFPrintfL(fp, "<c r=\"%s%d\">\n", osCol.c_str(), iRow);
                     VSIFPrintfL(fp, "<v>%d</v>\n", poFeature->GetFieldAsInteger(j));
                     VSIFPrintfL(fp, "</c>\n");
                 }
                 else if (eType == OFTInteger64)
                 {
-                    VSIFPrintfL(fp, "<c r=\"%s%d\">\n", szCol, iRow);
+                    VSIFPrintfL(fp, "<c r=\"%s%d\">\n", osCol.c_str(), iRow);
                     VSIFPrintfL(fp, "<v>" CPL_FRMT_GIB "</v>\n", poFeature->GetFieldAsInteger64(j));
                     VSIFPrintfL(fp, "</c>\n");
                 }
@@ -2106,7 +2105,7 @@ static bool WriteLayer(const char* pszName, OGRLayer* poLayer, int iLayer,
                     int s = (eType == OFTDate) ? 1 : (eType == OFTDateTime) ? 2 : 3;
                     if( eType == OFTDateTime && OGR_GET_MS(fSecond) )
                         s = 4;
-                    VSIFPrintfL(fp, "<c r=\"%s%d\" s=\"%d\">\n", szCol, iRow, s);
+                    VSIFPrintfL(fp, "<c r=\"%s%d\" s=\"%d\">\n", osCol.c_str(), iRow, s);
                     if (eType != OFTTime)
                         dfNumberOfDaysSince1900 += NUMBER_OF_DAYS_BETWEEN_1900_AND_1970;
                     if (eType == OFTDate)
@@ -2128,7 +2127,7 @@ static bool WriteLayer(const char* pszName, OGRLayer* poLayer, int iLayer,
                         oStringMap[pszVal] = nStringIndex;
                         oStringList.push_back(pszVal);
                     }
-                    VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"s\">\n", szCol, iRow);
+                    VSIFPrintfL(fp, "<c r=\"%s%d\" t=\"s\">\n", osCol.c_str(), iRow);
                     VSIFPrintfL(fp, "<v>%d</v>\n", nStringIndex);
                     VSIFPrintfL(fp, "</c>\n");
                 }
